/*
 * Copyright (c) 2003-2005 Jinyang Li
 *                    Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/


#include "math.h"
#include "p2psim/topology.h"
#include "p2psim/network.h"
#include "protocols/protocolfactory.h"
#include "e2eds2graph.h"
#include <iostream>

using namespace std;

E2EDS2Graph::E2EDS2Graph(vector<string> *v)
{
  _num = atoi((*v)[0].c_str());
  assert(_num > 0);
}

E2EDS2Graph::~E2EDS2Graph()
{
}

Time
E2EDS2Graph::latency(IPAddress ip1x, IPAddress ip2x, bool reply)
{
  IPAddress ip1 = Network::Instance()->first_ip(ip1x);
  IPAddress ip2 = Network::Instance()->first_ip(ip2x);

  if(!(ip1 > 0 && ip1 <= _num) ||
    (!(ip2 > 0 && ip2 <= _num)))
  {
    cout << "ip1 = " << ip1 << ", ip2 = " << ip2 << endl;
  }

  assert(ip1 > 0 && ip1 <= _num);
  assert(ip2 > 0 && ip2 <= _num);
  int t;
  t = (int)getSyntheticDistance((int)(ip1-1), (int)(ip2-1));
  return (Time)t;
}

bool
E2EDS2Graph::valid_latency (IPAddress ip1x, IPAddress ip2x)
{
  IPAddress ip1 = Network::Instance()->first_ip(ip1x);
  IPAddress ip2 = Network::Instance()->first_ip(ip2x);

  assert(ip1 > 0 && ip1 <= _num);
  assert(ip2 > 0 && ip2 <= _num);
  float t = getSyntheticDistance((int)(ip1-1), (int)(ip2-1));

  return (t > 0);
}

void
E2EDS2Graph::parse(ifstream &ifs)
{
  string line;
  vector<Time> tmp;

  while(getline(ifs,line)) {
    vector<string> words = split(line);
    // skip empty lines and commented lines
    if(words.empty() || words[0][0] == '#')
      continue;

    // XXX Thomer:  The else clause also seems
    // to expect 2 words per line?
    if (words[0] == "node") {
      IPAddress ipaddr = atoi(words[1].c_str());
      assert(ipaddr > 0 && ipaddr <= _num);

      // what kind of node?
      Node *p;
      if (words.size()<=2) {
	p = ProtocolFactory::Instance()->create(ipaddr);
      }else{
	p = ProtocolFactory::Instance()->create(ipaddr,words[2].c_str());
      }

      // add the node to the network
      send(Network::Instance()->nodechan(), &p);


    } else {
      printf("\n\nBad format in topology file!\n\n");
      break;
    }
  }

  //Assuming the name of the model file is DS2Model
  loadModel("DS2Model");
}

void E2EDS2Graph::loadModel(char* model_file_name)
{
	FILE *model_file;
	int NUM;
	int DIM;
	int BLOCKNUM;
	int MAXI;
	int HEAD_NUM;
	int MAX_ROW;
	
	int i,j;
	
	model_file = fopen(model_file_name,"r");
	assert(model_file != NULL);

	fscanf(model_file, "%d%d%d%d%d%d", &NUM, &DIM, &BLOCKNUM, &MAXI, &HEAD_NUM, &MAX_ROW);
	this->NUM_NODES = NUM;

	coords = (float **)malloc(NUM *sizeof(float*));
	for(i=0;i<NUM;i++)
		coords[i] = (float*)malloc(DIM*sizeof(float));
	for(i=0;i<NUM;i++)
		for(j=0;j<DIM;j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			coords[i][j] = v;
		}
	


	mapping = (int *) malloc(NUM*sizeof(int));
	for(i=0;i<NUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		mapping[i] = (int)v;
	}
	
	
	maxi_all = (int *)malloc(sizeof(int) * BLOCKNUM);
	for(i=0;i<BLOCKNUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		maxi_all[i] = (int)v;
	}


	p1_all = (float**)malloc(BLOCKNUM * sizeof(float*));
	p2_all = (float**)malloc(BLOCKNUM * sizeof(float*));
	pOverlap_all = (float**)malloc(BLOCKNUM * sizeof(float*));
	for(i=0;i<BLOCKNUM;i++)
	{
		p1_all[i] = (float*)malloc(maxi_all[i] * sizeof(float));
		p2_all[i] = (float*)malloc(maxi_all[i] * sizeof(float));
		pOverlap_all[i] = (float*)malloc(maxi_all[i] * sizeof(float));
	}
	

	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			p1_all[i][j] = v;
		}
	
	
	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			p2_all[i][j] = v;
		}
		
	
	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			pOverlap_all[i][j] = v;
		}
	



	heads = (int*)malloc(sizeof(int) * NUM);
	R = (float*)malloc(HEAD_NUM * sizeof(float));
	B = (float*)malloc(HEAD_NUM * sizeof(float));
	for(i=0;i<NUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		heads[i] = (int)v;
	}
	

	for(i=0;i<HEAD_NUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		R[i] = v;
	}


	for(i=0;i<HEAD_NUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		B[i] = v;
	}


	dis1_all = (float**)malloc(MAX_ROW * sizeof(float*));
	dis2_all = (float**)malloc(MAX_ROW * sizeof(float*));
	for(i=0;i<MAX_ROW;i++)
	{
		dis1_all[i] = (float*)malloc(300*sizeof(float));
		dis2_all[i] = (float*)malloc(300*sizeof(float));
	}
	
	
	for(i=0;i<MAX_ROW;i++)
		for(j=0;j<300;j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			dis1_all[i][j] = v;
		}
	

	for(i=0;i<MAX_ROW;i++)
		for(j=0;j<300;j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			dis2_all[i][j] = v;
		}

}

float E2EDS2Graph::getSyntheticDistance(int id1, int id2)
{
		float result;
		int clusterID1,clusterID2;
		int blockID;
		int tmp;

		if(id1 == id2)
			return 0;
		
		clusterID1 = mapping[id1];
		clusterID2 = mapping[id2];
		
		tmp = (clusterID1 <= clusterID2) ? clusterID1 : clusterID2;
		clusterID2 = (clusterID1>=clusterID2) ? clusterID1 : clusterID2;
		clusterID1 = tmp;
		

		if(clusterID1 == 1 && clusterID2 == 1)
			blockID = 1;
		else if(clusterID1 ==1 && clusterID2 == 2)
			blockID = 2;
		else if(clusterID1 ==1 && clusterID2 == 3)
			blockID = 3;
		else if(clusterID1 ==1 && clusterID2 == 4)
			blockID = 4;
		else if(clusterID1 ==2 && clusterID2 == 2)
			blockID = 5;
		else if(clusterID1 ==2 && clusterID2 == 3)
			blockID = 6;
		else if(clusterID1 ==2 && clusterID2 ==4)
			blockID = 7;
		else if(clusterID1 ==3 && clusterID2 ==3)
			blockID = 8;
		else if(clusterID1 ==3 && clusterID2 ==4)
			blockID = 9;
		else blockID = 10;
		
		blockID = blockID-1;
		
		int i,j,k;
		int sum_maxi_all[10];
		sum_maxi_all[0] = 0;
		for(i=1;i<10;i++)
			sum_maxi_all[i] = sum_maxi_all[i-1]+maxi_all[i-1];
	
		
		float sum;
		sum = 0;
		for(i=0;i<5;i++)
		{
			sum += pow(coords[id1][i]-coords[id2][i],2);
		}
		
		int v;
		float v_f;
		v_f = sqrt(sum);
		v = (int)v_f;
		
		if(v == 0 || v>=maxi_all[blockID])
		      return v_f*1000;
	
		assert(v>0);
		float prob1, prob2;
		prob1 = p1_all[blockID][v];
		prob2 = prob1 + p2_all[blockID][v];
		
		if(v<2000)
		{   
			srand( id1*id2);
			float r = rand()/(float)RAND_MAX;
			
			if(r >= prob2-pOverlap_all[blockID][v])
			{
				return v_f*1000;
			}
			
			r = rand()/(float)RAND_MAX;
			
			float **dis;
			if(r<prob1/prob2)
				dis = dis1_all;
			else dis = dis2_all;
			
		
			float rr = rand()/(float)RAND_MAX;
			for(k=0;k<300;k++)
				if(dis[sum_maxi_all[blockID]+v][k] > rr)
					break;
				
			if(k>95 && k<=100)
			{
				return v_f*1000;
			}
			int index;
			index = k-1;
			if(index == -1)
				index = 0;
			

			int err;
			if(index<100)
				err = index;
			else if(index >=299)
				err = (int)(-200 + rand()/(float)RAND_MAX *(-300));
			else err = -1 * (index-99);
			
			v_f = v_f - v_f*err/100.0;
			v = (int)v_f;

			//smooth here
			r = rand()/(float)RAND_MAX;
			if(r<0.333)
				v = (v-1)>0?(v-1):v;
			else if(r>0.667)
				v = v+1;

			if(v>2000)
			{
				v_f = 1000 + rand()/(float)RAND_MAX*1000;
				v = (int)v_f;
			}

			//start local distortion
			if(heads[id1] ==0 && heads[id2] == 0)
			{
				return v_f*1000;
			}

			int id1_tmp,id2_tmp;
			id1_tmp = id1;
			id2_tmp = id2;
			if(heads[id1] == 0)
			{
				id1_tmp = id2;
				id2_tmp = id1;
			}
			if(heads[id1]==1 && heads[id2]==1)
			{
				float r = rand()/(float)RAND_MAX;
				if(r>0.5)
				{
					id1_tmp = id1>id2?id1:id2;
					id2_tmp = id1<id2?id1:id2;	
				}
				else
				{
					id1_tmp = id1>id2?id2:id1;
					id2_tmp = id1<id2?id2:id1;
				}
			}

			
			float rad = R[id1_tmp];
			float bi = B[id1_tmp];
			
			if(v_f<=rad && v_f>bi)
				result = bi*0.99;
			else  result = v_f;
			
		}
		else result = 1000+rand()/(float)RAND_MAX*1000; 
		
		return (result+MIN_RTT)*1000;

}
#include "p2psim/bighashmap.cc"
