/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

#include <iostream>
#include <unistd.h>
#include "Utility.h"
#include "Vivaldi.h"
#include "Synthesizer.h"
#include "Analyzer.h"
#include "Gnp.h"


int usage()
{
  fprintf(stderr, "Usage of syn:\n [-a Euclidean embedding algorithm: 0 for GNP and 1 for Vivaldi (default).]\n [-m Name of the measured delay matrix (mat_real as default).]\n [-n Size of measured delay matrix.You must provide the size of matrix when using the tool.]\n [-s Scaling factor for synthesizer, it must be larger than 1 and default value is set to 10.]\n [-r Type 1 triangle inequality violation ratio threshold, it is in (0, 100) and default value is 85.]\n [-i Type 2 triangle inequality violation ratio threshold, it is in (100, Infinity) and default value is 250.]\n [-t Cutoff value for global clustering algorithm, default value is 120ms.]\n [-l Number of neighbors, default value is 32. In GNP algorithm, it is the number of landmarks; while in Vivaldi algorithm, it is the number of neighbors.]\n [-h Help information (this message).]\n\n");

  exit(-1);
}


int main (int argc, char * const argv[]) {

	CAnalyzer analyzer;
	CSynthesizer synthesizer;
	CGnp gnp;
	int i,j;
	int ch;

	if(argc == 1)
		usage();
	
	char* coordsName;
	float **coords;

	char oriMatName[200]; 
	strcpy(oriMatName, "mat_real");
	int size = 0;
	int dim = 5;
	int scalingFactor = 10;
	int embed_algo = 1;//0:GNP, 1:Vivaldi
	int R = 85;
	int iR = 250;
	int num_lm = 32;
	int cluster_threshold = 120;

	while ((ch = getopt(argc, argv, "a:n:s:r:i:t:m:l:h")) != -1)
  	{
    	 switch (ch)
    	 {
           case 'a':
        	embed_algo = atoi(optarg);
        	break;
           case 'n':
        	size = atoi(optarg);
        	break;
           case 's':
        	scalingFactor = atoi(optarg);
        	break;
           case 'r':
        	R = atoi(optarg);
        	break;
           case 'i':
        	iR = atoi(optarg);
        	break;
	   case 't':
		cluster_threshold = atoi(optarg);
		break;
	   case 'm':
		strcpy(oriMatName, optarg);
		break;
	   case 'l':
		num_lm = atoi(optarg);
		break;
      	   case 'h':
      	   default:
        	usage();
    	}//end of switch
       }//end of while
	
       if(size <= 0 || embed_algo>1 || scalingFactor<1 || R>=100 || R<=0 ||iR<=100 ||cluster_threshold<=0 || num_lm<=5 || num_lm>=size-1)
      		usage();

       printf(" Name of measured matrix: %s\n Size of measured matrix: %d\n Embedding algorithm is %s\n scalingFactor = %d\n R=%d iR=%d\n Clustering threshold = %d\n number of neighbors = %d\n\n", oriMatName, size, embed_algo==0?"GNP":"Vivaldi", scalingFactor, R, iR, cluster_threshold, num_lm);


	//compute coordinates
	printf("Calculating coordinates...\n");
	coordsName = "coord_ori";
	if(embed_algo == 1)
	{
	//case1: computing coordinates using vivaldi algorithm
		coords = vivaldiCoords_distributed(oriMatName, coordsName, size, dim, num_lm);
		CUtility::writeMatrix(coordsName, coords, size, dim);
	}
	else
	{
	//case2: compute coordiantes using GNP algorithm
	//default algorithm
	FILE *combined_file;
	combined_file = CUtility::FOPEN("combined", "w");//generating an empty file named combined

	char* gnp_conf_name = "conf";
        gnp.gen_conf(gnp_conf_name,size,num_lm,5,1,1);
	gnp.gnp(size, 0, gnp_conf_name, oriMatName, "combined");
	char* file_names[2];
	char model_file_name[20];
	char remain_file_name[20];
	
	sprintf(model_file_name, "model.%s.0", gnp_conf_name);
	sprintf(remain_file_name, "remain.%s.0", gnp_conf_name);
	file_names[0] = model_file_name;
	file_names[1] = remain_file_name;
	extract_coords(dim, 2, file_names);
	coords = CUtility::readMatrix(coordsName, size, dim);
	}
	printf("Coordinates calculation done\n");
        
	//adjust coordinates
	//coords = CUtility::readMatrix("gnp_coord_ori", size, dim);//temporary
	printf("Adjusting coordinates...\n");
	coordsName = "coords_ori_aa";//after adjustment
	analyzer.adjustCoords(oriMatName, coords, coordsName, size, dim, 10, 15);
    	CUtility::freeMatrix(coords, size);
	coords = CUtility::readMatrix(coordsName, size, dim);
	printf("Coordinates adjustment done\n");

	//compute predicted matrix
	//coordsName = "coords_ori_aa";//temporary
	printf("Generating embedded matrix...\n");
	char *predictedMatName = "mat_embeded";
	analyzer.getMatrix(coordsName, predictedMatName, size, dim);

	
	//calculate relative error
	printf("Calculating relative error matrix...\n");
	char* relErrMatName = "mat_relErr";
	analyzer.calculateRelErr(oriMatName, predictedMatName, relErrMatName, size);

	
	//calculate triangle inequality violation
	printf("Calculating triangle inequality violations ratio matrices...\n");
	char* aveRatioMatName = "mat_aveRatio";
	char* aveRatioSmaMatName = "mat_aveRatio_sma";
	analyzer.calculateTriangleViolations(oriMatName, aveRatioMatName, aveRatioSmaMatName, size);
	
	
	//do KNN analysis
	printf("Start KNN analysis...\n");
	IntVector ind_vec;
	IntVector center_id;
	analyzer.KNNAnalysis(oriMatName, size, 1, ind_vec, center_id);
	float slope = analyzer.calculateSlope(ind_vec);
	printf("slope:%f\n", slope);

	//do global clustering
	printf("Start doing global clustering...\n");
	IntVector C1ID, C2ID, C3ID, C4ID;
	analyzer.clustering(oriMatName, size, cluster_threshold, C1ID, C2ID, C3ID, C4ID);
	
	
	//reorganizing coordinates to make it ready for synthesis
	printf("Re-organizing coordinates for synthesis...\n");
	int th;
	int th_h1, th_h2, th_h3, th_l1, th_l2, th_l3;
	th = center_id.size();
	IntVector iv_h1, iv_h2, iv_h3, iv_h4;
	IntVector iv_l1, iv_l2, iv_l3, iv_l4;
	IntVector leaves_id, iv_all;
	for(i=0; i<size; i++)
		iv_all.push_back(i);
	
	leaves_id = CUtility::setDiff(iv_all, center_id);
	CUtility::writeIntVector("leaves",leaves_id);

	iv_h1 = CUtility::intersect(C1ID, center_id);
	iv_h2 = CUtility::intersect(C2ID, center_id);
	iv_h3 = CUtility::intersect(C3ID, center_id);
	iv_h4 = CUtility::intersect(C4ID, center_id);
	iv_l1 = CUtility::intersect(C1ID, leaves_id);
	iv_l2 = CUtility::intersect(C2ID, leaves_id);
	iv_l3 = CUtility::intersect(C3ID, leaves_id);
	iv_l4 = CUtility::intersect(C4ID, leaves_id);
	

	th_h1 = iv_h1.size();
	th_h2 = iv_h2.size()+th_h1;
	th_h3 = iv_h3.size()+th_h2;
	th_l1 = iv_l1.size()+th;
	th_l2 = iv_l2.size()+th_l1;
	th_l3 = iv_l3.size()+th_l2;
	//printf("%d %d %d %d %d %d\n", th_h1, th_h2, th_h3, th_l1, th_l2, th_l3);

	IntVector nodesInOrder = CUtility::concat_iv(CUtility::concat_iv(CUtility::concat_iv(iv_h1,iv_h2), CUtility::concat_iv(iv_h3,iv_h4)),CUtility::concat_iv(CUtility::concat_iv(iv_l1,iv_l2), CUtility::concat_iv(iv_l3,iv_l4)));
	//CUtility::writeIntVector("nodesInOrder", nodesInOrder);
    	float **coords_tmp;
	coords_tmp = CUtility::mallocMatrix(size, dim);
	for(i=0; i<size; i++)
		for(j=0; j<dim; j++)
		coords_tmp[i][j] = coords[nodesInOrder[i]][j];
	CUtility::freeMatrix(coords, size);
	coords = coords_tmp;

	
	//reorganizing matrices
	printf("Re-organizing matrices for statistics extraction...\n");
	char oriMatReorgaName[100];
	sprintf(oriMatReorgaName, "%s%s", oriMatName, "_reorga");
	
	char aveRatioReorgaName[100];
	sprintf(aveRatioReorgaName, "%s%s", aveRatioMatName, "_reorga");
	char aveRatioSmaReorgaName[100];
	sprintf(aveRatioSmaReorgaName, "%s%s", aveRatioSmaMatName, "_reorga");
	char relErrReorgaName[100];
	sprintf(relErrReorgaName, "%s%s", relErrMatName, "_reorga");
	char predictedMatReorgaName[100];
	sprintf(predictedMatReorgaName, "%s%s", predictedMatName, "_reorga");
        printf("start reorganizing matrices\n");

	analyzer.reorgaMatrix(oriMatName, oriMatReorgaName, size, C1ID, C2ID, C3ID, C4ID);
	analyzer.reorgaMatrix(aveRatioMatName, aveRatioReorgaName, size, C1ID, C2ID, C3ID, C4ID);
	analyzer.reorgaMatrix(aveRatioSmaMatName, aveRatioSmaReorgaName, size, C1ID, C2ID, C3ID, C4ID);
	analyzer.reorgaMatrix(relErrMatName, relErrReorgaName, size, C1ID, C2ID, C3ID, C4ID);
	analyzer.reorgaMatrix(predictedMatName, predictedMatReorgaName, size, C1ID, C2ID, C3ID, C4ID);

	
	printf("Start Euclidian map synthesis...\n");
	//do synthesis
	char* coords_sim_name = "coords_sim";
	synthesizer.synth(coords, size, coords_sim_name, 0, scalingFactor+10, th, th_h1, th_h2, th_h3, th_l1, th_l2, th_l3);
	synthesizer.synth(coords, size, coords_sim_name, 1, scalingFactor+10, th, th_h1, th_h2, th_h3, th_l1, th_l2, th_l3);
	
	//assembly the coordinates to get coords_syn;
	printf("Assembly all synthesis coordinates together...\n");
	float **coords_syn;
	float **coords_h1, **coords_h2, **coords_h3, **coords_h4, **coords_l1, **coords_l2, **coords_l3, **coords_l4;
	float **coords_0, **coords_1, **coords_t;
	int size1, size2;
	
	
	coords_0 = CUtility::readCoords("coords_h_1_0", dim, size1);
	coords_1 = CUtility::readCoords("coords_h_1_1", dim, size2);
	coords_h1 = CUtility::concat_coords(coords_0, size1, coords_1, size2, dim);
	coords_t = coords_h1;
	assert(size1+size2 >= scalingFactor*iv_h1.size());
	coords_h1 = CUtility::sample_coords(coords_h1, size1+size2, CUtility::randVector(size1+size2, scalingFactor*iv_h1.size(), 0), dim);
	CUtility::freeMatrix(coords_t, size1+size2);
	CUtility::writeMatrix("coords_h_1",coords_h1,scalingFactor*iv_h1.size(),dim);


	coords_0 = CUtility::readCoords("coords_h_2_0", dim, size1);
	coords_1 = CUtility::readCoords("coords_h_2_1", dim, size2);
	coords_h2 = CUtility::concat_coords(coords_0, size1, coords_1, size2, dim);
	coords_t = coords_h2;
	assert(size1+size2 >= scalingFactor*iv_h2.size());
	coords_h2 = CUtility::sample_coords(coords_h2, size1+size2, CUtility::randVector(size1+size2, scalingFactor*iv_h2.size(), 0), dim);
	CUtility::freeMatrix(coords_t, size1+size2);
	CUtility::writeMatrix("coords_h_2",coords_h2,scalingFactor*iv_h2.size(),dim);

	coords_0 = CUtility::readCoords("coords_h_3_0", dim, size1);
	coords_1 = CUtility::readCoords("coords_h_3_1", dim, size2);
	coords_h3 = CUtility::concat_coords(coords_0, size1, coords_1, size2, dim);
	coords_t = coords_h3;
	assert(size1+size2 >= scalingFactor*iv_h3.size());
	coords_h3 = CUtility::sample_coords(coords_h3, size1+size2, CUtility::randVector(size1+size2, scalingFactor*iv_h3.size(), 0), dim);
	CUtility::freeMatrix(coords_t, size1+size2);
	CUtility::writeMatrix("coords_h_3",coords_h3,scalingFactor*iv_h3.size(),dim);

	coords_0 = CUtility::readCoords("coords_h_4_0", dim, size1);
	coords_1 = CUtility::readCoords("coords_h_4_1", dim, size2);
	coords_h4 = CUtility::concat_coords(coords_0, size1, coords_1, size2, dim);
	coords_t = coords_h4;
	assert(size1+size2 >= scalingFactor*iv_h4.size());
	coords_h4 = CUtility::sample_coords(coords_h4, size1+size2, CUtility::randVector(size1+size2, scalingFactor*iv_h4.size(), 0), dim);
	CUtility::freeMatrix(coords_t, size1+size2);
	CUtility::writeMatrix("coords_h_4",coords_h4,scalingFactor*iv_h4.size(),dim);

	coords_0 = CUtility::readCoords("coords_l_1_0", dim, size1);
	coords_1 = CUtility::readCoords("coords_l_1_1", dim, size2);
	coords_l1 = CUtility::concat_coords(coords_0, size1, coords_1, size2, dim);
	coords_t = coords_l1;
	assert(size1+size2 >= scalingFactor*iv_l1.size());
	coords_l1 = CUtility::sample_coords(coords_l1, size1+size2, CUtility::randVector(size1+size2, scalingFactor*iv_l1.size(), 0), dim);
	CUtility::freeMatrix(coords_t, size1+size2);
	CUtility::writeMatrix("coords_l_1",coords_l1,scalingFactor*iv_l1.size(),dim);

	coords_0 = CUtility::readCoords("coords_l_2_0", dim, size1);
	coords_1 = CUtility::readCoords("coords_l_2_1", dim, size2);
	coords_l2 = CUtility::concat_coords(coords_0, size1, coords_1, size2, dim);
	coords_t = coords_l2;
	assert(size1+size2 >= scalingFactor*iv_l2.size());
	coords_l2 = CUtility::sample_coords(coords_l2, size1+size2, CUtility::randVector(size1+size2, scalingFactor*iv_l2.size(), 0), dim);
	CUtility::freeMatrix(coords_t, size1+size2);
	CUtility::writeMatrix("coords_l_2",coords_l2,scalingFactor*iv_l2.size(),dim);

	coords_0 = CUtility::readCoords("coords_l_3_0", dim, size1);
	coords_1 = CUtility::readCoords("coords_l_3_1", dim, size2);
	coords_l3 = CUtility::concat_coords(coords_0, size1, coords_1, size2, dim);
	coords_t = coords_l3;
	assert(size1+size2 >= scalingFactor*iv_l3.size());
	coords_l3 = CUtility::sample_coords(coords_l3, size1+size2, CUtility::randVector(size1+size2, scalingFactor*iv_l3.size(), 0), dim);
	CUtility::freeMatrix(coords_t, size1+size2);
	CUtility::writeMatrix("coords_l_3",coords_l3,scalingFactor*iv_l3.size(),dim);

	coords_0 = CUtility::readCoords("coords_l_4_0", dim, size1);
	coords_1 = CUtility::readCoords("coords_l_4_1", dim, size2);
	coords_l4 = CUtility::concat_coords(coords_0, size1, coords_1, size2, dim);
	coords_t = coords_l4;
	assert(size1+size2 >= scalingFactor*iv_l4.size());
	coords_l4 = CUtility::sample_coords(coords_l4, size1+size2, CUtility::randVector(size1+size2, scalingFactor*iv_l4.size(), 0), dim);
	CUtility::freeMatrix(coords_t, size1+size2);
	CUtility::writeMatrix("coords_l_4",coords_l4,scalingFactor*iv_l4.size(),dim);

	
	//calculate p1,p2,dis2,dis2,poverlap
	printf("Calculating global distortion statistics...\n");
	float **mat_relErr;
	float **mat_aveRatio;
	float **mat_aveRatio_sma;
	float **mat_embed;
	mat_relErr = CUtility::readMatrix(relErrReorgaName, size, size);
	mat_aveRatio = CUtility::readMatrix(aveRatioReorgaName, size, size);
	mat_aveRatio_sma = CUtility::readMatrix(aveRatioSmaReorgaName, size, size);
	mat_embed = CUtility::readMatrix(predictedMatReorgaName, size, size);

	IntVector size_clusters;
	size_clusters.push_back(C1ID.size());
	size_clusters.push_back(C2ID.size());
	size_clusters.push_back(C3ID.size());
	size_clusters.push_back(C4ID.size());
	analyzer.calculatePDis(mat_relErr, mat_aveRatio, mat_aveRatio_sma, mat_embed, size, size_clusters, R, iR, 1);
	CUtility::freeMatrix(mat_relErr, size);
	CUtility::freeMatrix(mat_aveRatio, size);
	CUtility::freeMatrix(mat_aveRatio_sma, size);
	CUtility::freeMatrix(mat_embed, size);
	
	//calculate heads, mapping
	printf("Extracting local distortion statistics...\n");
	IntVector heads_num_per_cluster;
	IntVector leaves_num_per_cluster;
	heads_num_per_cluster.push_back(iv_h1.size());
	heads_num_per_cluster.push_back(iv_h2.size());
	heads_num_per_cluster.push_back(iv_h3.size());
	heads_num_per_cluster.push_back(iv_h4.size());
	leaves_num_per_cluster.push_back(iv_l1.size());
	leaves_num_per_cluster.push_back(iv_l2.size());
	leaves_num_per_cluster.push_back(iv_l3.size());
	leaves_num_per_cluster.push_back(iv_l4.size());
	analyzer.calculateHeadsMapping(heads_num_per_cluster, leaves_num_per_cluster, scalingFactor, "heads", "mapping");
    	
	
	//calculate R, B
	float **coords_tmp1;
	float **coords_tmp2;
	coords_tmp1 = CUtility::concat_coords(coords_h1,iv_h1.size()*scalingFactor,coords_h2,iv_h2.size()*scalingFactor,dim);
	coords_tmp2 = CUtility::concat_coords(coords_h3,iv_h3.size()*scalingFactor,coords_h4,iv_h4.size()*scalingFactor,dim);
	coords_syn = CUtility::concat_coords(coords_tmp1,(iv_h1.size()+iv_h2.size())*scalingFactor,coords_tmp2,(iv_h3.size()+iv_h4.size())*scalingFactor,dim);
	coords_tmp1 = CUtility::concat_coords(coords_l1,iv_l1.size()*scalingFactor,coords_l2,iv_l2.size()*scalingFactor,dim);
	coords_tmp2 = CUtility::concat_coords(coords_l3,iv_l3.size()*scalingFactor,coords_l4,iv_l4.size()*scalingFactor,dim);
	coords_tmp1 = CUtility::concat_coords(coords_tmp1,(iv_l1.size()+iv_l2.size())*scalingFactor,coords_tmp2,(iv_l3.size()+iv_l4.size())*scalingFactor,dim);
        coords_syn = CUtility::concat_coords(coords_syn,(iv_h1.size()+iv_h2.size()+iv_h3.size()+iv_h4.size())*scalingFactor,coords_tmp1,(iv_l1.size()+iv_l2.size()+iv_l3.size()+iv_l4.size())*scalingFactor,dim);
        
	CUtility::writeMatrix("coords_syn", coords_syn, size*scalingFactor, dim);
	
	IntVector iv_heads;
	for(i=0; i<th*scalingFactor; i++)
		iv_heads.push_back(i);
	analyzer.calculateRB(coords_syn, size*scalingFactor, dim, iv_heads, 30, "R", "B", slope);

	//DS2 static analysis stage done here!
	
	
	//assembly all files into one file (the delay space model)
	FILE *model_file;
	model_file = CUtility::FOPEN("DS2Model","w");
	float **data;
	IntVector iv;
	iv = CUtility::loadIntVector("maxi");
	
	int NUM = size*scalingFactor;
	int BLOCKNUM = 10;
	int NUM_HEADS = th*scalingFactor;
	int NUM_ROWS = CUtility::sum_row(iv);
	fprintf(model_file,"%d %d %d %d %d %d\n", NUM, dim, BLOCKNUM, CUtility::max_row(iv), NUM_HEADS, NUM_ROWS);
	fflush(model_file);
	CUtility::writeMatrix(model_file, coords_syn, NUM, dim);
	
	CUtility::freeMatrix(coords_syn, NUM);

	data = CUtility::readMatrix("mapping", NUM, 1);
	CUtility::writeMatrix(model_file, data, NUM,1);
	
	data = CUtility::readMatrix("maxi", 1, BLOCKNUM);
	CUtility::writeMatrix(model_file, data, 1, BLOCKNUM);
	
	FILE *fin;
	fin = CUtility::FOPEN("p1","r");
	for(i=0;i<BLOCKNUM;i++)
	{
		for(j=0;j<iv[i];j++)
		{
			float v;
			fscanf(fin,"%f",&v);
			fprintf(model_file,"%f ", v);
		}
		fprintf(model_file,"\n");
	}
	fflush(model_file);
	fclose(fin);

	fin = CUtility::FOPEN("p2","r");
	for(i=0;i<BLOCKNUM;i++)
	{
		for(j=0;j<iv[i];j++)
		{
			float v;
			fscanf(fin,"%f",&v);
			fprintf(model_file,"%f ", v);
		}
		fprintf(model_file,"\n");
	}
	fflush(model_file);
	fclose(fin);

	fin = CUtility::FOPEN("pOverlap","r");
	for(i=0;i<BLOCKNUM;i++)
	{
		for(j=0;j<iv[i];j++)
		{
			float v;
			fscanf(fin,"%f",&v);
			fprintf(model_file,"%f ", v);
		}
		fprintf(model_file,"\n");
	}
	fflush(model_file);
	fclose(fin);
	
	data = CUtility::readMatrix("heads", NUM, 1);
	CUtility::writeMatrix(model_file, data, NUM, 1);
	
	data = CUtility::readMatrix("R", NUM_HEADS, 1);
	CUtility::writeMatrix(model_file, data, NUM_HEADS, 1);

	data = CUtility::readMatrix("B", NUM_HEADS, 1);
	CUtility::writeMatrix(model_file, data, NUM_HEADS, 1);
	
	data = CUtility::readMatrix("dis1", NUM_ROWS, 300);
	CUtility::writeMatrix(model_file, data, NUM_ROWS, 300);
	CUtility::freeMatrix(data, NUM_ROWS);
	
	data = CUtility::readMatrix("dis2", NUM_ROWS, 300);
	CUtility::writeMatrix(model_file, data, NUM_ROWS, 300);
	CUtility::freeMatrix(data, NUM_ROWS);
	
	
	printf("Deleting temporary files...\n");
	remove("B");
	remove("combined");
	remove("conf");
	remove("coords_syn");
	remove("coords_h_1");
	remove("coords_h_2"); 
	remove("coords_h_3");
	remove("coords_h_4");
	remove("coords_l_1");
	remove("coords_l_2");
	remove("coords_l_3");
	remove("coords_l_4");
	remove("coords_h_1_0");
	remove("coords_h_1_1");
        remove("coords_h_2_0");
	remove("coords_h_2_1");
	remove("coords_h_3_0");
	remove("coords_h_3_1");
	remove("coords_h_4_0");
	remove("coords_h_4_1");
	remove("coords_l_1_0");
	remove("coords_l_1_1");
	remove("coords_l_2_0");
	remove("coords_l_2_1");
	remove("coords_l_3_0");
	remove("coords_l_3_1");
	remove("coords_l_4_0");
	remove("coords_l_4_1");
	remove("coords_sim");
	remove("coords_ori_aa");
	remove("coord_ori");
	remove("dis1");
	remove("dis2");
	remove("heads");
	remove("heads_sim");
	remove("leaves");
	remove("leaves_sim");
	//remove(argv[1]);
	remove("mapping");
	remove("mat_aveRatio");
	remove("mat_aveRatio_sma");
	remove("mat_aveRatio_reorga");
	remove("mat_aveRatio_sma_reorga");
	remove("mat_embeded");
	remove("mat_embeded_reorga");
	remove("mat_relErr");
	remove("mat_relErr_reorga");
	remove("maxi");
	remove("model.conf.0");
	remove("p1");
	remove("p2");
	remove("pOverlap");
	remove("R");
	remove("remain.conf.0");
	char tmp[1000];
	sprintf(tmp,"%s%s",oriMatName,"_reorga");
	remove(tmp);

	fclose(model_file);
	return 0;
}
